@@ -54,33 +54,7 @@ struct _GMountTracker
   DBusConnection *connection;
 };
 
-/* We use this hack to avoid problems when this code
-   is shared in both the daemon and the client */
-static GType _g_mount_tracker_get_type (void) G_GNUC_CONST;
-
-#define g_mount_tracker_get_type _g_mount_tracker_get_type
 G_DEFINE_TYPE (GMountTracker, g_mount_tracker, G_TYPE_OBJECT)
-#undef g_mount_tracker_get_type
-
-GType
-g_mount_tracker_get_type (void)
-{
-  static volatile gsize type_volatile = 0;
-  
-  if (g_once_init_enter (&type_volatile))
-    {
-      GType type;
-      
-      type = g_type_from_name ("GMountTracker");
-      if (type == 0)
-	type = _g_mount_tracker_get_type ();
-      
-      g_once_init_leave (&type_volatile, type);
-    }
-  
-  return type_volatile;
-}
-
 
 static DBusHandlerResult g_mount_tracker_filter_func  (DBusConnection        *conn,
 						       DBusMessage           *message,
@@ -115,7 +89,8 @@ g_mount_info_dup (GMountInfo *info)
   copy->ref_count = 1;
   copy->display_name = g_strdup (info->display_name);
   copy->stable_name = g_strdup (info->stable_name);
-  copy->icon = g_strdup (info->icon);
+  copy->x_content_types = g_strdup (info->x_content_types);
+  copy->icon = g_object_ref (info->icon);
   copy->dbus_id = g_strdup (info->dbus_id);
   copy->object_path = g_strdup (info->object_path);
   copy->mount_spec = g_mount_spec_copy (info->mount_spec);
@@ -140,7 +115,8 @@ g_mount_info_unref (GMountInfo *info)
     {
       g_free (info->display_name);
       g_free (info->stable_name);
-      g_free (info->icon);
+      g_free (info->x_content_types);
+      g_object_unref (info->icon);
       g_free (info->dbus_id);
       g_free (info->object_path);
       g_mount_spec_unref (info->mount_spec);
@@ -184,11 +160,14 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
   dbus_bool_t user_visible;
   char *display_name;
   char *stable_name;
-  char *icon;
+  char *x_content_types;
+  char *icon_str;
   char *prefered_filename_encoding;
   char *dbus_id;
   char *obj_path;
   char *fuse_mountpoint;
+  GIcon *icon;
+  GError *error;
 
   if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_STRUCT)
     return NULL;
@@ -200,7 +179,8 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
 				      DBUS_TYPE_OBJECT_PATH, &obj_path,
 				      DBUS_TYPE_STRING, &display_name,
 				      DBUS_TYPE_STRING, &stable_name,
-				      DBUS_TYPE_STRING, &icon,
+                                      DBUS_TYPE_STRING, &x_content_types,
+				      DBUS_TYPE_STRING, &icon_str,
 				      DBUS_TYPE_STRING, &prefered_filename_encoding,
 				      DBUS_TYPE_BOOLEAN, &user_visible,
 				      G_DBUS_TYPE_CSTRING, &fuse_mountpoint,
@@ -213,11 +193,23 @@ g_mount_info_from_dbus (DBusMessageIter *iter)
     return NULL;
   }
 
+  if (icon_str == NULL || strlen (icon_str) == 0)
+    icon_str = "drive-removable-media";
+  error = NULL;
+  icon = g_icon_new_for_string (icon_str, &error);
+  if (icon == NULL)
+    {
+      g_warning ("Malformed icon string '%s': %s", icon_str, error->message);
+      g_error_free (error);
+      icon = g_themed_icon_new ("gtk-missing-image"); /* TODO: maybe choose a better name */
+    }
+
   info = g_new0 (GMountInfo, 1);
   info->ref_count = 1;
   info->display_name = g_strdup (display_name);
   info->stable_name = g_strdup (stable_name);
-  info->icon = g_strdup (icon);
+  info->x_content_types = g_strdup (x_content_types);
+  info->icon = icon;
   info->dbus_id = g_strdup (dbus_id);
   info->object_path = g_strdup (obj_path);
   info->mount_spec = mount_spec;
