@@ -158,6 +158,7 @@ struct _MetaTree {
   int fd;
   char *data;
   gsize len;
+  ino_t inode;
 
   guint32 tag;
   gint64 time_t_base;
@@ -438,6 +439,7 @@ meta_tree_init (MetaTree *tree)
 
 	  dir = g_path_get_dirname (tree->filename);
 	  g_mkdir_with_parents (dir, 0700);
+	  g_free (dir);
 
 	  builder = meta_builder_new ();
 	  retried = TRUE;
@@ -468,6 +470,7 @@ meta_tree_init (MetaTree *tree)
 
   tree->fd = fd;
   tree->len = statbuf.st_size;
+  tree->inode = statbuf.st_ino;
   tree->data = data;
   tree->header = (MetaFileHeader *)data;
 
@@ -613,12 +616,24 @@ meta_tree_unref (MetaTree *tree)
 static gboolean
 meta_tree_needs_rereading (MetaTree *tree)
 {
+  struct stat statbuf;
+
   if (tree->fd == -1)
     return TRUE;
 
   if (tree->header != NULL &&
       GUINT32_FROM_BE (tree->header->rotated) == 0)
     return FALSE; /* Got a valid tree and its not rotated */
+
+  /* Sanity check to avoid infinite loops when a stable file
+     has the rotated bit set to 1 (see gnome bugzilla bug #600057) */
+
+  if (lstat (tree->filename, &statbuf) != 0)
+    return FALSE;
+
+  if (tree->inode == statbuf.st_ino)
+    return FALSE;
+
   return TRUE;
 }
 
@@ -896,7 +911,7 @@ verify_journal_entry (MetaJournal *journal,
   if (entry_len != entry_len_end)
     return NULL;
 
-  real_crc32 = crc32 (journal->data + offset + 8, entry_len - 8);
+  real_crc32 = metadata_crc32 (journal->data + offset + 8, entry_len - 8);
   if (real_crc32 != GUINT32_FROM_BE (entry->crc32))
     return NULL;
 
@@ -1019,7 +1034,7 @@ meta_journal_entry_finish (GString *out)
   len = out->len + 4;
   append_uint32 (out, len);
   set_uint32 (out, 0, len);
-  set_uint32 (out, 4, crc32 (out->str + 8, len - 8));
+  set_uint32 (out, 4, metadata_crc32 (out->str + 8, len - 8));
   return out;
 }
 
@@ -1909,6 +1924,7 @@ meta_tree_enumerate_dir (MetaTree                         *tree,
 	break;
     }
  out:
+  g_free (res_path);
   g_hash_table_destroy (children);
   g_static_rw_lock_reader_unlock (&metatree_lock);
 }
@@ -2156,6 +2172,7 @@ meta_tree_enumerate_keys (MetaTree                         *tree,
 
     }
  out:
+  g_free (res_path);
   g_hash_table_destroy (keys);
   g_static_rw_lock_reader_unlock (&metatree_lock);
 }
@@ -2334,6 +2351,7 @@ static gboolean
 meta_tree_flush_locked (MetaTree *tree)
 {
   MetaBuilder *builder;
+  gboolean res;
 
   builder = meta_builder_new ();
 
@@ -2342,12 +2360,14 @@ meta_tree_flush_locked (MetaTree *tree)
   if (tree->journal)
     apply_journal_to_builder (tree, builder);
 
-  meta_builder_write (builder,
-		      meta_tree_get_filename (tree));
+  res = meta_builder_write (builder,
+			    meta_tree_get_filename (tree));
+  if (res)
+    meta_tree_refresh_locked (tree);
 
-  meta_tree_refresh_locked (tree);
+  meta_builder_free (builder);
 
-  return FALSE;
+  return res;
 }
 
 gboolean
@@ -2996,7 +3016,10 @@ update_mountinfo (void)
   contents = read_contents (mountinfo_fd);
   lseek (mountinfo_fd, SEEK_SET, 0);
   if (contents)
-    mountinfo_roots = parse_mountinfo (contents);
+    {
+      mountinfo_roots = parse_mountinfo (contents);
+      g_free (contents);
+    }
 }
 
 static char *
@@ -3081,7 +3104,8 @@ find_mountpoint_for (MetaLookupCache *cache,
   last = g_strdup (file);
   while (1)
     {
-      dir_dev = get_devnum (dir);
+      if (dir)
+	dir_dev = get_devnum (dir);
       if (dir == NULL ||
 	  dev != dir_dev)
 	{
@@ -3116,7 +3140,7 @@ find_mountpoint_for (MetaLookupCache *cache,
  */
 static char *
 expand_all_symlinks (const char *path,
-		     dev_t *dev_out)
+		     dev_t      *dev_out)
 {
   char *parent, *parent_expanded;
   char *basename, *res;
@@ -3137,6 +3161,7 @@ expand_all_symlinks (const char *path,
       g_free (parent_expanded);
       g_free (basename);
       g_free (parent);
+      g_free (path_copy);
     }
   else
     res = path_copy;
@@ -3161,6 +3186,7 @@ meta_lookup_cache_free (MetaLookupCache *cache)
   g_free (cache->last_parent_expanded);
   g_free (cache->last_parent_mountpoint);
   g_free (cache->last_parent_mountpoint_extra_prefix);
+  g_free (cache->last_device_tree);
   g_free (cache);
 }
 
@@ -3212,6 +3238,7 @@ expand_parents (MetaLookupCache *cache,
       strcmp (cache->last_parent, parent) != 0)
     {
       g_free (cache->last_parent);
+      g_free (cache->last_parent_expanded);
       cache->last_parent = parent;
       cache->last_parent_expanded = expand_all_symlinks (parent, &parent_dev);
       cache->last_parent_dev = parent_dev;
@@ -3225,6 +3252,7 @@ expand_parents (MetaLookupCache *cache,
 
   *parent_dev_out = cache->last_parent_dev;
   basename = g_path_get_basename (path_copy);
+  g_free (path_copy);
   res = g_build_filename (cache->last_parent_expanded, basename, NULL);
   g_free (basename);
 
@@ -3305,6 +3333,7 @@ meta_lookup_cache_lookup_path (MetaLookupCache *cache,
     }
 
  found:
+  g_free (expanded);
   tree = meta_tree_lookup_by_name (treename, for_write);
   if (tree)
     {
